(ns com.oly.static-sites.do-blog.pages.chess-board
  (:require
   [com.oly.static-sites.do-blog.pages.chess.movement
    :refer [fetch-selected-piece-moves valid-move?]]
   [com.oly.static-sites.do-blog.pages.chess.render
    :refer [calculate-black-white]]
   [com.oly.static-sites.do-blog.pages.chess.state
    :refer [game-state start-state]]
   [com.oly.static-sites.ui-hiccup.interface :refer [board]]
   [reagent.core :as r]))

(defonce game-state-viking (r/atom []))

(defn board-test [size]
  (loop [x-pos 1
         y-pos 1
         result []]
    (if (and (= x-pos size) (= y-pos size))
      result
      (if (= size x-pos)
        (recur 1
               (inc y-pos)
               (conj result [x-pos y-pos]))
        (recur (inc x-pos)
               y-pos
               (conj result [x-pos y-pos]))))))

(defn square-defaults [pos]
  {:class (if (= (calculate-black-white pos) :black)
            "bg-slate-200 black-80"
            "bg-slate-800 white-80")})

(defn get-square [pos]
  (get @game-state [:board pos] {}))

(defn reset-square
  ([pos] (reset-square pos (square-defaults pos)))
  ([pos value]
   (swap! game-state assoc-in [:board pos]  value)))

(defn update-square [pos value]
  (swap! game-state update-in [:board pos]
         merge #_(get-in @game-state [:board pos] {})
         value))

(defn create-board [size]
  (->> (board-test size)
       (mapv #(let [square-colour (calculate-black-white %)]
                (vector % (square-defaults %))))
       (into {})))

(defn collisions [positions]
  (print "collissions")
  (print positions)
  (filterv (fn [[distance x y]]
             (let [piece (get-square [x y])]
               (prn (str "collision = " [x y] " - " (:type piece) " - " (:colour piece)))
               (and (nil? (:type piece))
                    (nil? (:colour piece))))) positions))

(defn highlight-moves [pos]
  (->> (fetch-selected-piece-moves
        (first (filter (fn [[k v]] (:selected v)) (-> @game-state :board)))
        (-> @game-state :board))
       (mapv (fn [[distance x y]]
               (update-square
                [x y]
                {:highlight (str " bg-green-" (+ 2 distance) "00 ")
                 :pos [x y]})))))

(defn clear-keys [key]
  (swap! game-state
         assoc :board
         (reduce-kv (fn [m pos piece]
                      (assoc m pos (dissoc piece key))) {} (-> @game-state :board))))

(defn get-selected-square []
  (last (filter (fn [[_ square]] (true? (:selected square))) (-> @game-state :board))))

(defn piece-attribute-match? [src-piece dest-piece #_#_src-pos dest-pos attrib]
  (if (and (:type dest-piece) '(= (get src-piece attrib) (get dest-piece attrib)))
    true false)
  #_(let [src-piece (get-square src-pos)
          dest-piece (get-square dest-pos)]))

(defn take-piece? [piece-one piece-two]
  (prn "Take piece --")
  (if (not= (:colour piece-one) (:colour piece-two))
    true false))

(defn move-piece
  [selected-piece dest-square]
  ;; check the colours don't match can't land on our own piece
  (if (piece-attribute-match? selected-piece dest-square :colour)
    nil
    (when (valid-move? (:pos dest-square) (-> @game-state :board))
      (prn (take-piece? selected-piece dest-square))
      (when (take-piece? selected-piece dest-square)
        (prn (str "taking " (:colour selected-piece)))
        (if (= :white (:colour selected-piece))

          (do (prn "white")
              (reset-square (:pos dest-square))
              (swap! game-state update :player-one conj dest-square))
          (do (prn "black")
              (reset-square (:pos dest-square))
              (swap! game-state update :player-two conj dest-square)
              )))

;(take-piece selected-piece dest-square)
      (update-square (:pos dest-square) selected-piece)
      (reset-square (:pos selected-piece) (square-defaults (:pos selected-piece))))))

(defn click-square [evt p]
  (let [[selected-pos selected-piece] (get-selected-square)
        clicked-square p]
    (clear-keys :highlight)

    (when selected-pos
      (do (prn "previously selected piece")
          (move-piece selected-piece clicked-square #_#_#_(:pos selected-piece) selected-piece (:pos p))
          #_(clear-keys :selected))
      #_(do
        (prn "no previously selected piece")
        (clear-keys :selected)
        (update-square (:pos p) (assoc p :selected true))
        (highlight-moves (:pos p)))))

  (clear-keys :selected)
  (clear-keys :highlight)
  ;; set new square as selected
  (update-square (:pos p) (assoc p :selected true))
  (highlight-moves (:pos p))
  (prn (get (-> @game-state :board) (:pos p))))

(defn enhance-board [board]
  (reduce-kv
   (fn [m key square]
     (assoc m key (assoc square :on-click click-square :pos key)))
   {}
   board))

(defn place-pieces-on-board [board pieces]
  (reduce (fn [m piece]
            (let [square-colour (calculate-black-white (:start-pos piece))]
              (assoc m (:start-pos piece)
                     (-> piece
                         (assoc :pos (:start-pos piece)
                                :text (:character piece)
                                :selected false
                                :on-click click-square
                                :class (if (= square-colour :black)
                                         (str "bg-slate-200 " (if (= :white (:colour piece))
                                                                " black "
                                                                " white "))

                                         (str "bg-slate-800 " (if (= :white (:colour piece)) " black" " white "))))))))

          board
          pieces))

(reset! game-state-viking
        (-> (create-board 11)))

(reset! game-state
        {:player-one []
         :player-two []
         :board
         (-> (create-board 8)
             (enhance-board)
             (place-pieces-on-board start-state))})

(defn chess-page [{:keys [path-params] :as route}]
  (fn [{:keys [path-params] :as route}]
    [:div
     (let [[pos piece] (get-selected-square)]
       [:div
        [:div "Player one " (str (mapv :type (-> @game-state :player-one )))]
        [:div "Player one " (str (-> @game-state :player-one ))]
        [:div "Player two " (str (mapv :type (-> @game-state :player-two ))) ]
        [:div (str "pos = " pos)]
        [:div (str (:type piece))]
        [:div (str piece)]
        [:div (str "move = " (fetch-selected-piece-moves
                              [pos piece]
                              (-> @game-state :board)))]])

     [:div (board 8 (-> @game-state :board))]
     [:div "Viking chess board"]
     [:div (board 11 @game-state-viking)]]))

(comment
  (get-selected-square)
  @game-state)


